
#include "usb910conf.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbcdc.h"
#include "usb910.h"


#define AUTO_RANGE_SUPPORT
#define BIPOLAR_RANGE_SUPPORT

#define poll    usbcdc_poll

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))


#ifdef OSC_EXTENSION

#define OSC_PORT       PORTB
#define OSC_DDR        DDRB
#define OSC_PIN        PB1

static uint8_t osc_stat;
static uint8_t clk_shift;
static uint16_t clk_counter;

static inline void osc_stop() {
	TCCR1B = 0; // stop clock
	TCCR1A = 0;
	if (osc_stat != OSC_INIT) {
		osc_stat = OSC_STOP;
	}
}

void init_osc() {
	osc_stat = OSC_INIT;
}

void term_osc() {
	bit_clear(OSC_PORT, OSC_PIN);
	bit_clear(OSC_DDR, OSC_PIN);
	osc_stop();
	osc_stat = OSC_INIT;
}


static void osc_start() {
	uint16_t cnt = clk_counter - 1;
	bit_set(OSC_PORT, OSC_PIN);
	bit_set(OSC_DDR, OSC_PIN);
	TCCR1B = 0; // stop clock first
	TC1H = cnt >> 8;
	OCR1C = cnt;
	TCCR1A = (1<<COM1A0); // toggle mode
	TCCR1B = clk_shift + 1;
	osc_stat = OSC_START;
}

union fn {
	uint32_t dw;
	struct {
		uint8_t b0;
		uint8_t b1;
		uint8_t b2;
		uint8_t b3;
	};
};

static inline int8_t osc_set(uint8_t *buf) {
	union fn fn;
	uint32_t fbase,fcnt;
	uint8_t i;
	fn.b0 = *buf++;
	fn.b1 = *buf++;
	fn.b2 = *buf++;
	fn.b3 = 0;
	if (fn.dw < 1 ) {
		return -1;
	} else if (fn.dw >= F_CPU/2/4) { /* MAX CLOCK */
		clk_shift = 0;
		clk_counter = 4;
		return 0;
	} else {
		fbase = F_CPU / 2;
		for (i=0; i< 15; i++) {
			fcnt = fbase / fn.dw;
			if (fcnt < 1024) {
				clk_shift = i;
				clk_counter = fcnt;
				return 0;
			}
			fbase >>= 1;
		}
	}
	return -1;
}

int8_t osc_send(uint8_t size) {
	if (size >= 1 && ext_buf[0] == OSC_START) {
		if (osc_stat == OSC_INIT) {
			return -2;
		}
		osc_start();
		return 1;
	} else if (size >= 1 && ext_buf[0] == OSC_STOP) {
		osc_stat = OSC_STOP;
		return 1;
	} else if (size >= 4 && ext_buf[0] == OSC_SET) {
		uint8_t save_stat = osc_stat;
		if (osc_set(ext_buf+1)) {
			return -3;
		}
		if (save_stat == OSC_START) {
			osc_start();
		} else {
			osc_stat = OSC_STOP;
		}
		return 4;
	} else return -2;
}

int8_t osc_recv(uint8_t size) {
	if (size >= 4) {
		union fn fn;
		if (osc_stat == OSC_INIT) {
			return -3;
		}
		fn.dw = F_CPU/2;
		fn.dw >>= clk_shift;
		fn.dw /= clk_counter;
		ext_buf[0] = osc_stat;
		ext_buf[1] = fn.b0;
		ext_buf[2] = fn.b1;
		ext_buf[3] = fn.b2;
		return 4;
	} else return -2;
}

#endif
