###############################################################################
# Makefile for the project rccal
###############################################################################

## General Flags
PROJECT = rccal44
MCU = attiny44
TARGET = $(PROJECT).elf
CC = avr-gcc

## Options common to compile, link and assembly rules
COMMON = -mmcu=$(MCU)
#APP_END = 0x1000
#DATA_END = 0x800160

#COMMON += -DAPP_END=$(APP_END) -DDATA_END="($(DATA_END)-0x800000)"

## Compile options common for all C compilation units.
CFLAGS = $(COMMON)
CFLAGS += -Wall -DF_CPU=8000000UL  -Os -fsigned-char

## Assembly specific flags
ASMFLAGS = $(COMMON)
ASMFLAGS += -x assembler-with-cpp -Wa

## Linker flags
LDFLAGS = $(COMMON)
LDFLAGS +=  -Wl,-Map=$(PROJECT).map
#LDFLAGS += -Ttext $(APP_END) -Tdata $(DATA_END)

## Intel Hex file production flags
HEX_FLASH_FLAGS = -R .eeprom


## Include Directories
INCLUDES = -I. 

## Objects that must be built in order to link
OBJECTS = usi2cdev.o rccal.o 

## Objects explicitly added by the user
LINKONLYOBJECTS = 

## Build
all: $(TARGET) $(PROJECT).hex $(PROJECT).lss size

## Compile
usi2cdev.o: usi2cdev.c
	$(CC) $(INCLUDES) $(CFLAGS) -c  $<

rccal.o: rccal.c 
	$(CC) $(INCLUDES) $(CFLAGS) -c  $<
	$(CC) $(INCLUDES) $(CFLAGS) -S  $<

##Link
$(TARGET): $(OBJECTS)
	 $(CC) $(LDFLAGS) $(OBJECTS) $(LINKONLYOBJECTS) $(LIBDIRS) $(LIBS) -o $(TARGET)

vectconv: ../vectconv.c
	cc -o vectconv $<

%.hex: $(TARGET) 
	avr-objcopy -O ihex $(HEX_FLASH_FLAGS)  $< $@
#	./vectconv 0 13 $< > $(PROJECT)0.hex

%.lss: $(TARGET)
	avr-objdump -h -S $< > $@

size: ${TARGET}
	@echo
	@avr-size ${TARGET}

## Clean target
.PHONY: clean
clean:
	-rm -rf *.s $(OBJECTS) $(PROJECT).{elf,lss,map} $(PROJECT)0.hex  vectconv
